#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['dots']

package_data = \
{'': ['*']}

install_requires = \
['click ~= 7.0.0', 'pyyaml ~= 5.1.2']

setup(name='dots',
      version='0.0.1a0',
      description='dots: dotfiles made easy',
      author='Pradyun Gedam',
      author_email='pradyunsg@gmail.com',
      url='https://github.com/pradyunsg/dots',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='>=3.5',
     )
